using System;
using System.IO;
using System.Collections;
using Cairo;
using Gtk;
using Mono.Unix;

namespace PLDAbout
{
	public class Background
	{
		private readonly Surface combined = Surface.CreateForImage(Format.RGB24, 600, 400);

		// combine background image with overlay and some text
		public Background()
		{
			ImageSurface background = new ImageSurface(GetRandom(Path.Combine(PLDAbout.Path, "backgrounds")));
			ImageSurface overlay = new ImageSurface(Path.Combine(PLDAbout.Path, "front.png"));

			Graphics g = new Graphics(combined);

			background.Show(g, 0, 0);
			overlay.Show(g, 0, 0);

			g.SetSourceRGBA(1, 1, 1, 0.65);
			g.FontFace("Arial", FontSlant.Normal, FontWeight.Normal);
			g.FontSetSize(40);
			g.MoveTo(540, 210);
			g.Rotate(90*Math.PI/180);
			g.ShowText("PLD 2.0");
		}

		private string GetRandom(string path)
		{
			string[] files = Directory.GetFiles(path);

			return files[new Random().Next(files.Length)];
		}

		public void Show(Graphics g, int x, int y)
		{
			combined.Show(g, x, y);
		}

		public static implicit operator Surface(Background bg)
		{
			return bg.combined;
		}
	}

	public class Image : DrawingArea
	{
		Surface background;

		public Image(int x, int y, Surface s)
		{
			SetSizeRequest(x, y);
			background = s;
		}

		protected override bool OnExposeEvent (Gdk.EventExpose args)
		{
			Graphics g = Gdk.Graphics.CreateDrawable(args.Window);
			background.Show(g, 0, 0);

			return true;
		}
	}

	public class Canvas : DrawingArea
	{
		private readonly ImageSurface stork = new ImageSurface(System.IO.Path.Combine(PLDAbout.Path, "bociek.png"));
		private readonly LinearGradient MaskTop = new LinearGradient(0, 0, 0, 20);
		private readonly LinearGradient MaskBottom = new LinearGradient(0, 0, 0, 20);
		private Surface scroll = Surface.CreateForImage(Format.RGB24, 400, 345);
		private Surface ScrollTop = Surface.CreateForImage(Format.RGB24, 400, 20);
		private Surface ScrollMiddle = Surface.CreateForImage(Format.RGB24, 400, 305);
		private Surface ScrollBottom = Surface.CreateForImage(Format.RGB24, 400, 20);
		private readonly Surface bg;
		private int offset = -65;
		private int wait = 60;
		private readonly int maxoffset;
		private readonly DevList DevsActive;
		private readonly DevList DevsRetired;
		private readonly int step;
		private bool turbo = false;
		private int multiplier = 1;
		private readonly int max_multiplier = 8;
		private int turbo_counter;

		public Canvas(bool lowend, Surface back)
		{
			bg = back;

			DevsActive = new DevList("dev-active", 340, 25);
			DevsRetired = new DevList("dev-retired", 90+340+DevsActive.Height, 25);
			maxoffset = 140+340+DevsActive.Height+DevsRetired.Height;

			MaskTop.AddColorStop(0, new Color(0, 0, 0, 0));
			MaskTop.AddColorStop(1, new Color(0, 0, 0, 1));
			MaskBottom.AddColorStop(0, new Color(0, 0, 0, 1));
			MaskBottom.AddColorStop(1, new Color(0, 0, 0, 0));

			RenderScroll();
			RenderMaskedScroll();
			SetSizeRequest(400, 345);

			if(lowend)
				step = 2;
			else
				step = 1;

			GLib.Timeout.Add((uint)step*1000/24, new GLib.TimeoutHandler(Tick));
		}

		private bool Tick()
		{
			if(turbo && multiplier < max_multiplier)
				multiplier++;
			if(!turbo && multiplier > 1)
				multiplier--;

			if(wait>0)
				wait -= step;
			else
			{
				offset += step * multiplier;

				if(offset>maxoffset)
					offset =- 345;
			}

			RenderScroll();
			RenderMaskedScroll();
			QueueDraw();
			return true;
		}

		protected override bool OnExposeEvent (Gdk.EventExpose args)
		{
			Graphics g = Gdk.Graphics.CreateDrawable(args.Window);

			ScrollTop.Show(g, 0, 0);

			g.Translate(0, 20);
			ScrollMiddle.Show(g, 0, 0);

			g.Translate(0, 305);
			ScrollBottom.Show(g, 0, 0);

			if(turbo)
			{
				if(turbo_counter % 10 < 5)
				{
					g.SetSourceRGBA(1, 1, 1, 0.9);
					g.MoveTo(0, 0);
					g.LineTo(7.5, 10);
					g.LineTo(15, 0);
					g.LineTo(0, 0);
					g.Fill();

					g.MoveTo(380, 0);
					g.LineTo(380+7.5, 10);
					g.LineTo(380+15, 0);
					g.LineTo(380, 0);
					g.Fill();
				}
				turbo_counter++;
			}

			return true;
		}

		private void RenderMaskedScroll()
		{
			Graphics g;
		       
			// fade out part of scroll
			g = new Graphics(ScrollTop);

			bg.Show(g, 0, 0);
			g.SetSourceSurface(scroll, 0, 0);
			g.Mask(MaskTop);

			// normal part of scroll
			g = new Graphics(ScrollMiddle);

			g.Translate(0, -20);
			scroll.Show(g, 0, 0);

			// fade in part of scroll
			// first cut bottom part of scroll
			Surface tmp = Surface.CreateForImage(Format.RGB24, 400, 20);
			g = new Graphics(tmp);

			g.Translate(0, -325);
			scroll.Show(g, 0, 0);

			// then do fade in
			g = new Graphics(ScrollBottom);

			g.Save();
			g.Translate(0, -325);
			bg.Show(g, 0, 0);
			g.Restore();
			g.SetSourceSurface(tmp, 0, 0);
			g.Mask(MaskBottom);
		}

		private void RenderScroll()
		{
			Graphics g = new Graphics(scroll);

			bg.Show(g, 0, 0);

			if(offset < 84)		// bociek.png height
			{
				g.Save();
				g.Translate(200-137/2, -offset);
				stork.Show(g, 0, 0);
				g.Restore();
			}
			
			g.SetSourceRGB(1, 1, 1);
			g.FontFace("Arial", FontSlant.Normal, FontWeight.Bold);
			g.FontSetSize(36);
			ShowTextCentered(g, "PLD Linux Distribution", 200, 130-offset);

			g.FontSetSize(64);
			ShowTextCentered(g, "AC", 200, 200-offset);

			g.FontFace("Arial", FontSlant.Italic, FontWeight.Normal);
			g.FontSetSize(22);
			ShowTextCentered(g, Catalog.GetString("Brought to you by:"), 200, 300-offset);	

			g.FontFace("Arial", FontSlant.Normal, FontWeight.Normal);
			g.FontSetSize(16);
			RenderDevList(g, DevsActive, -offset);

			g.FontFace("Arial", FontSlant.Italic, FontWeight.Normal);
			g.FontSetSize(22);
			ShowTextCentered(g, Catalog.GetString("Retired developers:"), 200, 50+340+DevsActive.Height-offset);

			g.FontFace("Arial", FontSlant.Normal, FontWeight.Normal);
			g.FontSetSize(16);
			RenderDevList(g, DevsRetired, -offset);

			g.FontFace("Arial", FontSlant.Normal, FontWeight.Bold);
			g.FontSetSize(22);
			ShowTextCentered(g, Catalog.GetString("Thanks for using PLD!"), 200, 130+340+DevsActive.Height+DevsRetired.Height-offset);
		}

		private void RenderDevList(Graphics g, DevList l, int start)
		{
			foreach(DevList.Element e in l.List)
				ShowTextCentered(g, e.Developer, 200, start+e.Offset);
		}

		private void ShowTextCentered(Graphics g, string str, int x, int y)
		{
			if(y<-20 || y>400)
				return;

			TextExtents t = Graphicx.TextExtents(g.Handle, str);
			g.Save();
			g.Translate(x-t.Width/2, y);
			g.ShowText(str);
			g.Stroke();
			g.Restore();
		}

		public bool Turbo
		{
			set
			{
				// compensate for key repeat
				if(value == true && turbo == false)
					turbo_counter = 0;

				turbo = value;
			}
		}
	}

	public class PLDAbout
	{
		public static string Path = System.AppDomain.CurrentDomain.BaseDirectory;
		private Canvas canvas;

		private Surface[] cut(Surface background)
		{
			Surface[] s = new Surface[5];
			Graphics g;

			s[0] = Surface.CreateForImage(Format.RGB24, 60, 400);
			g = new Graphics(s[0]);
			background.Show(g, 0, 0);

			s[1] = Surface.CreateForImage(Format.RGB24, 400, 27);
			g = new Graphics(s[1]);
			g.Translate(-60, 0);
			background.Show(g, 0, 0);

			s[2] = Surface.CreateForImage(Format.RGB24, 400, 345);
			g = new Graphics(s[2]);
			g.Translate(-60, -27);
			background.Show(g, 0, 0);

			s[3] = Surface.CreateForImage(Format.RGB24, 400, 28);
			g = new Graphics(s[3]);
			g.Translate(-60, -372);
			background.Show(g, 0, 0);

			s[4] = Surface.CreateForImage(Format.RGB24, 140, 400);
			g = new Graphics(s[4]);
			g.Translate(-460, 0);
			background.Show(g, 0, 0);
			
			return s;
		}

		PLDAbout(bool lowend)
		{
			Window w = new Window(Catalog.GetString("About PLD"));

			w.Decorated = false;
			w.KeepAbove = true;
			w.Resizable = false;
			w.DeleteEvent += delegate(object o, DeleteEventArgs args) { Application.Quit(); };
			w.ButtonPressEvent += delegate(object o, ButtonPressEventArgs args) { Application.Quit(); };
			w.KeyPressEvent += HandleKeys;
			w.KeyReleaseEvent += HandleKeys;
			w.Events = Gdk.EventMask.ExposureMask | Gdk.EventMask.ButtonPressMask;
			w.SetPosition(WindowPosition.Center);

			Surface[] s = cut(new Background());

			HBox hb = new HBox(false, 0);
			Image i;

			i = new Image(60, 400, s[0]);
			hb.PackStart(i);

			VBox vb = new VBox(false, 0);

			i = new Image(400, 27, s[1]);
			vb.PackStart(i);

			canvas = new Canvas(lowend, s[2]);
			vb.PackStart(canvas);

			i = new Image(400, 28, s[3]);
			vb.PackStart(i);

			hb.PackStart(vb);

			i = new Image(140, 400, s[4]);
			hb.PackStart(i);

			w.Add(hb);

			w.IconList = new Gdk.Pixbuf[1] {new Gdk.Pixbuf(null, "icon64.png")};

			w.ShowAll();
		}

		[GLib.ConnectBefore()]
		private void HandleKeys(object o, KeyPressEventArgs args)
		{
			switch(args.Event.Key)
			{
				case Gdk.Key.Escape:
					Application.Quit();
					break;

				case Gdk.Key.space:
				case Gdk.Key.KP_Space:
					canvas.Turbo = true;
					break;

				default:
					break;
			}
		}

		private void HandleKeys(object o, KeyReleaseEventArgs args)
		{
			if(args.Event.Key == Gdk.Key.space || args.Event.Key == Gdk.Key.KP_Space)
				canvas.Turbo = false;
		}

		public static void Main(string[] args)
		{
			bool lowend=false;

			foreach(string s in args)
				if(s == "--low-end")
				{
					Console.WriteLine("Running at half framerate");
					lowend = true;
					break;
				}

			Application.Init();

			Catalog.Init("pld-about", "/usr/share/locale");
			new PLDAbout(lowend);

			Application.Run();
		}
	}
}
